/* Get various resource limits for the tests */

#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#include <sys/types.h>
#include <sys/sysctl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/mount.h>
#include <kvm.h>
#include <vm/vm_param.h>
#include <errno.h>
#include <err.h>

#include "stress.h"

int64_t
inodes(void)
{
	char path[MAXPATHLEN+1];
	struct statfs buf;

	sync();
	if (getcwd(path, sizeof(path)) == NULL)
		err(1, "getcwd()");

	if (statfs(path, &buf) < 0)
		err(1, "statfs(%s)", path);
	if (op->verbose > 2)
		printf("Free inodes on %s (%s): %jd\n", path, buf.f_mntonname, buf.f_ffree);
	return (buf.f_ffree);
}

int64_t
df(void)
{
	char path[MAXPATHLEN+1];
	struct statfs buf;

	if (getcwd(path, sizeof(path)) == NULL)
		err(1, "getcwd()");

	if (statfs(path, &buf) < 0)
		err(1, "statfs(%s)", path);
	if (buf.f_bavail > buf.f_blocks || buf.f_bavail < 0) {
		warnx("Corrupt statfs(%s). f_bavail = %jd!", path, buf.f_bavail);
		buf.f_bavail = 100;
	}
	if (op->verbose > 2)
		printf("Free space on %s: %jd Mb\n", path, buf.f_bavail * buf.f_bsize / 1024 / 1024);
	return (buf.f_bavail * buf.f_bsize);
}


int64_t
swap(void)
{
	struct xswdev xsw;
	size_t mibsize, size;
	int mib[16], n;
	int64_t sz;

	mibsize = sizeof mib / sizeof mib[0];
	sz = 0;

	if (sysctlnametomib("vm.swap_info", mib, &mibsize) == -1)
		err(1, "sysctlnametomib()");

	for (n = 0; ; ++n) {
		mib[mibsize] = n;
		size = sizeof xsw;
		if (sysctl(mib, mibsize + 1, &xsw, &size, NULL, 0) == -1)
			break;
		if (xsw.xsw_version != XSWDEV_VERSION)
			errx(1, "xswdev version mismatch");
		sz = sz + xsw.xsw_nblks - xsw.xsw_used;
	}
	if (errno != ENOENT)
		err(1, "sysctl()");

	if (op->verbose > 2)
		printf("Total free swap space %jd Mb\n",
			sz * getpagesize() / 1024 / 1024);

	return (sz * getpagesize());
}

unsigned long
usermem(void)
{
	unsigned long mem;
	size_t nlen = sizeof(mem);

	if (sysctlbyname("hw.usermem", &mem, &nlen, NULL, 0) == -1)
		err(1, "sysctlbyname() %s:%d", __FILE__, __LINE__);

	if (op->verbose > 2)
		printf("Total free user memory %lu Mb\n",
			mem / 1024 / 1024);

	return (mem);
}
