/* Regression test of using almost all available inodes.
   Problems found: 1) Incorrecf df information for mountpoint
   2) Livelock
 */

#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#include <sys/types.h>
#include <sys/sysctl.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/mount.h>
#include <kvm.h>
#include <vm/vm_param.h>
#include <errno.h>
#include <err.h>

int64_t
inodes(void)
{
	char path[MAXPATHLEN+1];
	struct statfs buf;

	sync();
	if (getcwd(path, sizeof(path)) == NULL)
		err(1, "getcwd()");

	if (statfs(path, &buf) < 0)
		err(1, "statfs(%s)", path);
	printf("Free inodes on %s: %jd\n", path, buf.f_ffree);
	return (buf.f_ffree);
}

int
main() {
	int fd, i, j;
	int64_t size;
	pid_t pid;
	char file[128];

	size = inodes() - 1000;
	pid = getpid();
	for (j = 0; j < size; j++) {
		sprintf(file,"p%05d.%05d", pid, j);
		if ((fd = open(file, O_CREAT | O_TRUNC | O_WRONLY, 0666)) == -1) {
			if (errno != EINTR) {
				warn("creat(%s)", file);
/*				printf("break out at %d, errno %d\n", j, errno);*/
				break;
			}
		}
		if (fd != -1 && close(fd) == -1)
			err(2, "close(%d)", j);

	}

	for (i = --j; i >= 0; i--) {
		sprintf(file,"p%05d.%05d", pid, i);
		if (unlink(file) == -1)
			err(3, "unlink(%s)", file);
		
	}
	return (0);
}
