#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/mount.h>
#include <err.h>
#include <errno.h>

#include "stress.h"

static char path[128];
static unsigned long size;

int
setup(int nb)
{
	int pct;
	int64_t in;

	umask(0);
	if (nb == 0) {
		in = inodes();
		size = in / op->incarnations;

		pct = 90;
		if (op->hog == 0)
			pct = random_int(1, 90);
		size = size / 100 * pct;

		if (size > 20000)
			size = 20000;	/* arbitrary limit number of files pr. dir */

		if (op->verbose > 1)
			printf("creat: Free inodes %jd, size %d * %lu = %lu\n",
				in, op->incarnations, size, size * op->incarnations);
		putval(size);
	} else {
		size = getval();
	}

	sprintf(path,"%s.%05d", getprogname(), getpid());
	if (mkdir(path, 0770) < 0)
		err(1, "mkdir(%s), %s:%d", path, __FILE__, __LINE__);

	if (chdir(path) == -1)
		err(1, "chdir(%s), %s:%d", path, __FILE__, __LINE__);

	return (0);
}

void
cleanup(void)
{
	(void)chdir("..");
	if (rmdir(path) == -1) {
		warn("rmdir(%s), %s:%d", path, __FILE__, __LINE__);
	}
	
}

int
test(void)
{
	int fd, i, j;
	pid_t pid;
	char file[128];

	pid = getpid();
	for (j = 0; j < size && done_testing == 0; j++) {
		sprintf(file,"p%05d.%05d", pid, j);
		if ((fd = creat(file, 0660)) == -1) {
			if (errno != EINTR) {
				warn("creat(%s)", file);
/*				printf("break out at %d, errno %d\n", j, errno);*/
				break;
			}
		}
		if (fd != -1 && close(fd) == -1)
			err(2, "close(%d)", j);

	}

	for (i = --j; i >= 0; i--) {
		sprintf(file,"p%05d.%05d", pid, i);
		if (unlink(file) == -1)
			err(3, "unlink(%s)", file);
		
	}

	return (0);
}
