/* $FreeBSD$ */

/* Test lockf(3) */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/wait.h>
#include <signal.h>
#include <errno.h>
#include <err.h>

#include <stress.h>

char file[128];
int fd;
pid_t	pid;

int
get(void) {
	int sem;
	if (lockf(fd, F_LOCK, 0) == -1)
		err(1, "lockf(%s, F_LOCK)", file);
	if (read(fd, &sem, sizeof(sem)) != sizeof(sem))
		err(1, "get: read(%d)", fd);
	if (lseek(fd, 0, SEEK_SET) == -1)
		err(1, "lseek");
	if (lockf(fd, F_ULOCK, 0) == -1)
		err(1, "lockf(%s, F_ULOCK)", file);
	return (sem);
}

void
incr(void) {
	int sem;
	if (lockf(fd, F_LOCK, 0) == -1)
		err(1, "lockf(%s, F_LOCK)", file);
	if (read(fd, &sem, sizeof(sem)) != sizeof(sem))
		err(1, "incr: read(%d)", fd);
	if (lseek(fd, 0, SEEK_SET) == -1)
		err(1, "lseek");
	sem++;
	if (write(fd, &sem, sizeof(sem)) != sizeof(sem))
		err(1, "incr: read");
	if (lseek(fd, 0, SEEK_SET) == -1)
		err(1, "lseek");
	if (lockf(fd, F_ULOCK, 0) == -1)
		err(1, "lockf(%s, F_ULOCK)", file);
}
int
setup(int nb)
{
        return (0);
}

void
cleanup(void)
{
}

int
test(void)
{
	int i;
	int sem = 0;

	sprintf(file, "lockf.0.%d", getpid());
	if ((fd = open(file,O_CREAT | O_TRUNC | O_RDWR, 0600)) == -1) 
		err(1, "creat(%s)", file);
	if (write(fd, &sem, sizeof(sem)) != sizeof(sem))
		err(1, "write");
	if (lseek(fd, 0, SEEK_SET) == -1)
		err(1, "lseek");

	pid = fork();
	if (pid == -1) {
		perror("fork");
		exit(2);
	}

	if (pid == 0) {	/* child */
		for (i = 0; i < 100; i++) {
			while ((get() & 1) == 0)
				;
			if (op->verbose > 2)
				printf("Child  %d, sem = %d\n", i, get()),
					fflush(stdout);
			incr();
		}
		exit(0);
	} else {	/* parent */
		for (i = 0; i < 100; i++) {
			while ((get() & 1) == 1)
				;
			if (op->verbose > 2)
				printf("Parent %d, sem = %d\n", i, get()),
					fflush(stdout);
			incr();
		}
	}
	close(fd);
	waitpid(pid, &i, 0);
	unlink(file);

        return (0);
}
