#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#include <sys/types.h>
#include <sys/sysctl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <err.h>

#include "stress.h"

static unsigned long size;


int
setup(int nb)
{
	long minsize, maxsize;
	int64_t in;

	if (nb == 0) {
		maxsize = 50000;
		if (op->hog == 0) {
			minsize = maxsize * 0.4;
			if (minsize > 1000)
				minsize = 1000;
		} else
			minsize = maxsize * 0.8;
		maxsize = maxsize * 0.85;

		size = random_int(minsize, maxsize);

		in = inodes();
		if (size > in * 0.49) {
			if (op->verbose > 1)
				printf("Limiting size from %lu to %jd due to number of inodes\n",
						size, in);
			size = in * 0.49;
		}

		size = size / op->incarnations;

		if (size > 200)
			size = 200;	/* To deep directories causes trouble for find */
		putval(size);
	} else
		size = getval();

	if (op->verbose > 1 && nb == 0)
		printf("setup: pid = %d, minsize = %ld, maxsize = %ld, "
			"size = %ld (total %ld)\n", getpid(), 
			minsize, maxsize, size, size * op->incarnations);
	return (0);
}

void
cleanup(void)
{
}

void
mkDir(char *path, int level) {
	char newPath[MAXPATHLEN + 1];

	if (mkdir(path, 0770) == -1) {
		warn("mkdir(%s), %s:%d", path, __FILE__, __LINE__);
		size = level;
	} else
		chdir(path);

	if (done_testing == 1)
		size = level;

	if (level < size) {
		sprintf(newPath,"d%d", level+1);
		mkDir(newPath, level+1);
	}
}

void
rmDir(char *path, int level) {
	char newPath[MAXPATHLEN + 1];

	if (level < size) {
		sprintf(newPath,"d%d", level+1);
		rmDir(newPath, level+1);
	}
	chdir ("..");
	if (rmdir(path) == -1) {
		err(1, "rmdir(%s), %s:%d", path, __FILE__, __LINE__);
	}
}

int
test(void)
{
	char path[MAXPATHLEN + 1];

	umask(0);
	sprintf(path,"p%05d.d%d", getpid(), 1);
	mkDir(path, 1);
	rmDir(path, 1);

	return (0);
}
