#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>
#include <string.h>
#include <err.h>

#include "stress.h"

static char path[MAXPATHLEN+1];
#define NB (1 * 1024 * 1024)

int bufsize;

static void
reader(void) {
	int fd;
	int i, n, *buf;

	if ((fd = open(path, O_RDWR, 0600)) < 0) {
		unlink(path);
		err(1, "open(%s)", path);
	}
	if ((buf = malloc(bufsize)) == NULL)
			err(1, "malloc(%d), %s:%d", bufsize, __FILE__, __LINE__);
	for (i = 0; i < NB; i+= bufsize) {
		if ((n = read(fd, buf, bufsize)) < 0)
			err(1, "read(), %s:%d", __FILE__, __LINE__);
		if (n == 0) break;
	}
	close(fd);
	free(buf);
	return;
}

static void
writer(void) {
	int i, *buf;
	int fd;

	if ((fd = open(path, O_RDWR, 0600)) < 0) {
		unlink(path);
		err(1, "open(%s)", path);
	}
	if ((buf = malloc(bufsize)) == NULL)
			err(1, "malloc(%d), %s:%d", bufsize, __FILE__, __LINE__);
	for (i = 0; i < bufsize / sizeof(int); i++)
		buf[i] = i;

	for (i = 0; i < NB; i+= bufsize) {
		if (write(fd, buf, bufsize) < 0) {
			err(1, "write(%d), %s:%d", fd,
					__FILE__, __LINE__);
		}
	}
	close(fd);
	free(buf);
	return;
}

int
setup(int nb)
{
	bufsize = 2 << random_int(2, 12);
	return (0);
}

void
cleanup(void)
{
}

int
test(void)
{
	pid_t pid;
	int i, status;

	for (i = 0; i < 100; i++) {
		if (sprintf(path, "fifo.%d.%d", getpid(), i) < 0)
			err(1, "sprintf()");
		if (mkfifo(path, 0600) < 0)
			err(1, "mkfifo(%s)", path);
	}
	for (i = 0; i < 100; i++) {
		if (sprintf(path, "fifo.%d.%d", getpid(), i) < 0)
			err(1, "sprintf()");
		if (unlink(path) < 0)
			err(1, "unlink(%s)", path);
	}


	if (sprintf(path, "fifo.%d", getpid()) < 0)
		err(1, "sprintf()");
	if (mkfifo(path, 0600) < 0)
		err(1, "mkfifo(%s)", path);

	if ((pid = fork()) == 0) {
		writer();
		exit(EXIT_SUCCESS);

	} else if (pid > 0) {
		reader();
		kill(pid, SIGINT);
		if (waitpid(pid, &status, 0) == -1)
			warn("waitpid(%d)", pid);
	} else 
		err(1, "fork(), %s:%d",  __FILE__, __LINE__);

	unlink(path);
	return (0);
}
