#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <err.h>

#include "stress.h"

#define NB (400 * 1024 * 1024)

int port;
int bufsize;
int sv[2];

static void
reader(void) {
	int n, t, *buf;

	t = 0;
	if ((buf = malloc(bufsize)) == NULL)
			err(1, "malloc(%d), %s:%d", bufsize, __FILE__, __LINE__);
	while (done_testing == 0) {
		if ((n = read(sv[0], buf, bufsize)) < 0)
			err(1, "read(), %s:%d", __FILE__, __LINE__);
		t += n;
		if (n == 0) break;
	}
	close(sv[0]);
	return;
}

static void
writer(void) {
	int i, *buf, r;

	if (r < 0)
		err(1, "connect(), %s:%d", __FILE__, __LINE__);

	if ((buf = malloc(bufsize)) == NULL)
			err(1, "malloc(%d), %s:%d", bufsize, __FILE__, __LINE__);
	for (i = 0; i < bufsize / sizeof(int); i++)
		buf[i] = i;

	for (;;) {
		for (i = 0; i < NB; i+= bufsize) {
			if (write(sv[1], buf, bufsize) < 0) {
				if (errno == EPIPE)
					return;
				err(1, "write(%d), %s:%d", sv[1],
						__FILE__, __LINE__);
			}
		}
	}
	return;
}

int
setup(int nb)
{
	port = 12340 + nb;
	bufsize = 2 << random_int(2, 12);
	return (0);
}

void
cleanup(void)
{
}

int
test(void)
{
	pid_t pid;

	if (socketpair(PF_UNIX, SOCK_STREAM, 0, sv) != 0)
		err(1, "socketpair()");
	if ((pid = fork()) == 0) {
		writer();
		exit(EXIT_SUCCESS);

	} else if (pid > 0) {
		reader();
		kill(pid, SIGINT);
	} else 
		err(1, "fork(), %s:%d",  __FILE__, __LINE__);

	return (0);
}
