#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#include <sys/types.h>
#include <sys/sysctl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <err.h>


#include "stress.h"

static unsigned long size;

int
setup(int nb)
{
	unsigned long minsize, maxsize;
	unsigned long mem;
	unsigned long swapinfo = 0;
	unsigned long s;
	struct rlimit rlp;

	if (nb == 0) {
		swapinfo = swap();
		mem = usermem();

		minsize = mem;
		if (op->hog == 0)
			maxsize = mem + swapinfo * 0.5;
		else
			maxsize = mem + swapinfo * 0.9;

		s = size = random_int(minsize, maxsize);
		size = size / op->incarnations;

		if (getrlimit(RLIMIT_DATA, &rlp) < 0)
			err(1,"getrlimit");
		rlp.rlim_cur -= 1024 * 1024;

		if (size > rlp.rlim_cur)
			size = rlp.rlim_cur;
		putval(size);


		if (op->verbose > 1 && nb == 0)
			printf("setup: pid %d, minsize %ldMb, maxsize %ldMb, "
					"size %ldMb (total %ldMb)\n", getpid(), 
				minsize/1024/1024, maxsize/1024/1024, size/1024/1024,
				size * op->incarnations / 1024 / 1024);
	} else
		size = getval();
	return (0);
}

void
cleanup(void)
{
}

int
test(void)
{
	char *c;
	int i, page;
	unsigned long oldsize = size;
	time_t start;

	c = malloc(size);
	while (c == NULL && done_testing == 0) {
		size -=  1024 * 1024;
		c = malloc(size);
	}
	if (op->verbose > 1 && size != oldsize)
		printf("Malloc size changed from %ld Mb to %ld Mb\n",
			oldsize / 1024 / 1024, size / 1024 / 1024);
	page = getpagesize();
	start = time(NULL);	/* Livelock workaround */
	while (done_testing == 0 &&
			(time(NULL) - start) < op->run_time) {
		i = 0;
		while (i < size && done_testing == 0) {
			c[i] = 0;
			i += page;
		}
#if 0
		if (op->hog != 1)
			usleep(1000);
#endif
	}
	free(c);

	return (0);
}
